import math
import os


def get_many(f, item_filter=None):
    items = []
    offset = 0
    size = 100
    while True:
        m = f(offset=offset, size=size)
        for item in m.items:
            if item_filter is None or item_filter(item):
                items += [item]
        if len(m.items) != size:
            break
        offset += size
        print(offset)
    return items


def download_sources_by_data(dest, contest, problems, participants, submissions):
    def lang_ext(lang):
        if ":" in lang:
            lang = lang[:lang.find(":")]
        if lang == "bf":
            return "bf"
        if lang == "csharp":
            return "cs"
        if lang == "d":
            return "d"
        if lang == "pascal":
            return "pas"
        if lang == "go":
            return "go"
        if lang in ["gpp", "cpp"]:
            return "cpp"
        if lang == "haskell":
            return "hs"
        if lang == "java":
            return "java"
        if lang == "js":
            return "js"
        if lang == "kotlin":
            return "kt"
        if lang == "lua":
            return "lua"
        if lang == "php":
            return "php"
        if lang in ["pypy", "python"]:
            return "py"
        if lang == "ruby":
            return "rb"
        if lang == "rust":
            return "rs"
        if lang == "c":
            return "c"
        if lang == "plain":
            return "txt"
        print('Language not found:', lang)
        return "txt"

    ends_at = contest.ends_at.seconds

    problems_dict = dict({})
    for problem in problems:
        problems_dict[problem.id] = chr(ord('a') + problem.index - 1)

    participants_dict = dict({})
    for participant in participants:
        participants_dict[participant.id] = participant.name

    for submission in submissions:
        if submission.submitted_at.seconds > ends_at:
            continue
        username = submission.participant_id

        if submission.participant_id in participants_dict:
            username = participants_dict[submission.participant_id]

        path = "{}/{}/{}/{}_{}.{}".format(
            dest,
            username,
            problems_dict[submission.problem_id],
            submission.submitted_at.seconds,
            math.floor(submission.percentage * 100),
            lang_ext(submission.lang)
        )

        os.makedirs(os.path.dirname(path), exist_ok=True)

        with open(path, "w") as file:
            file.write(submission.source)



from eolymp.judge import judge_pb2, acl_pb2
from eolymp.judge.acl_http import AclClient

from peolymp.judge import JudgeAPI
from peolymp.utils import download_sources_by_data


class JudgeContestAPI(JudgeAPI):

    def __init__(self, contest_id, **kwargs):
        super(JudgeContestAPI, self).__init__(**kwargs)
        self.contest_id = contest_id
        self.contest = self.client.DescribeContest(judge_pb2.DescribeContestInput(contest_id=contest_id)).contest
        self.client_acl = AclClient(self.http_client, url=self.contest.url)

    def add_admin(self, user_id):
        self.client_acl.GrantPermission(acl_pb2.GrantPermissionInput(user_id=user_id, role="ADMIN"))

    def add_participant(self, **kwargs):
        return super().add_participant(contest_id=self.contest_id, **kwargs)

    def add_problem(self, **kwargs):
        return super().add_problem(contest_id=self.contest_id, **kwargs)

    def sync_problem(self, **kwargs):
        return super().sync_problem(contest_id=self.contest_id, **kwargs)

    def sync_all_problems(self, **kwargs):
        return super().sync_all_problems(contest_id=self.contest_id, **kwargs)

    def lock_participant_by_id(self, **kwargs):
        return super().lock_participant_by_id(contest_id=self.contest_id, **kwargs)

    def get_contest(self, **kwargs):
        return super().get_contest(contest_id=self.contest_id, **kwargs)

    def delete_contest(self):
        return super().delete_contest(contest_id=self.contest_id)

    def get_problems(self):
        return super().get_problems(contest_id=self.contest_id)

    def get_scoring(self):
        return super().get_scoring(contest_id=self.contest_id)

    def allow_upsolve(self):
        return super().allow_upsolve(contest_id=self.contest_id)

    def get_participant(self, **kwargs):
        return super().get_participant(contest_id=self.contest_id, **kwargs)

    def get_participants(self):
        return super().get_participants(contest_id=self.contest_id)

    def get_submissions(self):
        return super().get_submissions(contest_id=self.contest_id)

    def rejudge_problem(self, **kwargs):
        return super().rejudge_problem(contest_id=self.contest_id, **kwargs)

    def download_sources(self, dest):
        return download_sources_by_data(dest, self.contest, self.get_problems(),
                                        self.get_participants(),
                                        self.get_submissions())



from eolymp.ranker import ranker_pb2, scoreboard_pb2, format_pb2
from eolymp.ranker.ranker_http import RankerClient
from eolymp.wellknown import direction_pb2

from peolymp.abstract import AbstractAPI
from peolymp.utils import get_many


class RankerAPI(AbstractAPI):
    def __init__(self, **kwargs):
        super(RankerAPI, self).__init__(**kwargs)
        self.client = RankerClient(self.http_client, url=self.get_url())

    def create_scoreboard(self, key, name, ioi=True):
        scoreboard = scoreboard_pb2.Scoreboard(key=key, name=name,
                                               format=format_pb2.Format.IOI if ioi else format_pb2.Format.ICPC)
        res = self.client.CreateScoreboard(ranker_pb2.CreateScoreboardInput(scoreboard=scoreboard))
        return res.scoreboard_id

    def add_attribute(self, scoreboard_id, attribute_key, key, name, full_name, filterable=True, sortable=False,
                      index=0):
        type_attribute = scoreboard_pb2.Scoreboard.Column.Type.ATTRIBUTE
        column = scoreboard_pb2.Scoreboard.Column(key=key, name=full_name, short_name=name, type=type_attribute,
                                                  community_attribute_key=attribute_key, index=index,
                                                  filterable=filterable, visible=True, sortable=sortable)
        return self.client.AddScoreboardColumn(
            ranker_pb2.AddScoreboardColumnInput(scoreboard_id=scoreboard_id, column=column))

    def add_total(self, scoreboard_id, key, name, full_name, filterable=True, sortable=False, index=0):
        type_total = scoreboard_pb2.Scoreboard.Column.Type.TOTAL
        column = scoreboard_pb2.Scoreboard.Column(key=key, name=full_name, short_name=name, type=type_total,
                                                  index=index, filterable=filterable, visible=True, sortable=sortable)
        return self.client.AddScoreboardColumn(
            ranker_pb2.AddScoreboardColumnInput(scoreboard_id=scoreboard_id, column=column)).column_id

    def add_name(self, scoreboard_id, key='name', short_name='Ім\'я', name='Ім\'я', index=1):
        type_name = scoreboard_pb2.Scoreboard.Column.Type.NAME
        column = scoreboard_pb2.Scoreboard.Column(key=key, name=name, short_name=short_name, type=type_name,
                                                  index=index)
        return self.client.AddScoreboardColumn(
            ranker_pb2.AddScoreboardColumnInput(scoreboard_id=scoreboard_id, column=column))

    def get_columns(self, scoreboard_id):
        return self.client.ListScoreboardColumns(ranker_pb2.ListScoreboardColumnsInput(scoreboard_id=scoreboard_id))

    def delete_column(self, column_id):
        return self.client.DeleteScoreboardColumn(ranker_pb2.DeleteScoreboardColumnInput(column_id=column_id))

    def add_contest(self, scoreboard_id, contest_scoreboard_id, key, name, full_name, parent_id="", index=0):
        type_contest = scoreboard_pb2.Scoreboard.Column.Type.CONTEST
        column = scoreboard_pb2.Scoreboard.Column(key=key, name=full_name, short_name=name, type=type_contest,
                                                  judge_contest_id=contest_scoreboard_id, index=index,
                                                  visible=True, sortable=True, parent_id=parent_id)
        return self.client.AddScoreboardColumn(
            ranker_pb2.AddScoreboardColumnInput(scoreboard_id=scoreboard_id, column=column)).column_id

    def rebuild(self, scoreboard_id):
        return self.client.RebuildScoreboard(ranker_pb2.RebuildScoreboardInput(scoreboard_id=scoreboard_id))

    def get_scoreboard(self, scoreboard_id):
        return self.client.DescribeScoreboard(
            ranker_pb2.DescribeScoreboardInput(scoreboard_id=scoreboard_id)).scoreboard

    def get_scoreboards(self):
        def __get_scoreboards(offset, size):
            return self.client.ListScoreboards(request=ranker_pb2.ListScoreboardsInput(offset=offset, size=size))

        return get_many(__get_scoreboards)

    def delete_scoreboard(self, scoreboard_id):
        return self.client.DeleteScoreboard(ranker_pb2.DeleteScoreboardInput(scoreboard_id=scoreboard_id))

    def delete_all_scoreboards(self):
        for scoreboard in self.get_scoreboards():
            self.delete_scoreboard(scoreboard.id)

    def make_default_sort_column(self, scoreboard_id, column_id):
        scoreboard = scoreboard_pb2.Scoreboard(id=scoreboard_id, default_sort_column=column_id,
                                               default_sort_order=direction_pb2.DESC)
        return self.client.UpdateScoreboard(
            ranker_pb2.UpdateScoreboardInput(scoreboard_id=scoreboard_id, scoreboard=scoreboard,
                                             patch=[ranker_pb2.UpdateScoreboardInput.DEFAULT_SORT]))



from peolymp.client import HTTPClient


class AbstractAPI:
    def __init__(self, username, password, space_id):
        self.space_id = space_id
        self.http_client = HTTPClient(username, password, space_id)

    def get_url(self):
        return "https://api.eolymp.com/spaces/" + self.space_id



import json

import requests
from eolymp.core.http_client import HttpClient


class HTTPClient(HttpClient):
    def __init__(self, username, password, space):
        resp = requests.post(url='https://api.eolymp.com/oauth/token',
                             data={'username': username, 'password': password,
                                   'grant_type': 'password'})
        token = json.loads(resp.text)['access_token']
        super().__init__(token=token, headers={'Space-ID': space})



from .atlas import AtlasAPI
from .cognito import CognitoAPI
from .community import CommunityAPI
from .judge import JudgeAPI
from .judge_contest import JudgeContestAPI
from .ranker import RankerAPI
from .typewriter import TypeWriterAPI
from .universe import UniverseAPI



from eolymp.typewriter import typewriter_pb2
from eolymp.typewriter.typewriter_http import TypewriterClient

from peolymp.abstract import AbstractAPI


class TypeWriterAPI(AbstractAPI):

    def __init__(self, **kwargs):
        super(TypeWriterAPI, self).__init__(**kwargs)
        self.client = TypewriterClient(self.http_client)

    def upload_asset(self, filename, data):
        return self.client.UploadAsset(typewriter_pb2.UploadAssetInput(filename=filename, data=data)).link



import time

from eolymp.judge import judge_pb2, contest_pb2
from eolymp.judge.judge_http import JudgeClient
from google.protobuf import timestamp_pb2

from peolymp.abstract import AbstractAPI
from peolymp.utils import get_many, download_sources_by_data


class JudgeAPI(AbstractAPI):

    def __init__(self, **kwargs):
        super(JudgeAPI, self).__init__(**kwargs)
        self.client = JudgeClient(self.http_client, url=self.get_url())

    def add_participant(self, member_id, contest_id):
        return self.client.AddParticipant(
            judge_pb2.AddParticipantInput(contest_id=contest_id, member_id=member_id)).participant_id

    def add_problem(self, problem_id, index, contest_id, submit_limit=50, score_by_best_testset=True):
        return self.client.ImportProblem(
            judge_pb2.ImportProblemInput(contest_id=contest_id, import_id=str(problem_id), index=index,
                                         submit_limit=submit_limit, score_by_best_testset=score_by_best_testset))

    def sync_problem(self, problem_id, contest_id):
        return self.client.SyncProblem(judge_pb2.SyncProblemInput(problem_id=problem_id, contest_id=contest_id))

    def sync_all_problems(self, contest_id, problem_number=-1):
        problems = self.get_problems(contest_id)
        counter = 0
        for problem in problems:
            if problem_number == -1 or counter == problem_number:
                self.sync_problem(problem.id, contest_id)
                time.sleep(5)
            counter += 1

    def lock_participant_by_id(self, participant_id, contest_id):
        out = self.client.ResetPasscode(
            judge_pb2.ResetPasscodeInput(participant_id=participant_id, contest_id=contest_id))
        return out.passcode

    def get_contest(self, contest_id):
        return self.client.DescribeContest(judge_pb2.DescribeContestInput(contest_id=contest_id)).contest

    def get_contests(self):
        def __get_contests(offset, size):
            return self.client.ListContests(request=judge_pb2.ListContestsInput(offset=offset, size=size))

        return get_many(__get_contests)

    def get_scoring(self, contest_id):
        return self.client.DescribeScoring(judge_pb2.DescribeScoringInput(contest_id=contest_id)).scoring

    def delete_contest(self, contest_id):
        self.client.DeleteContest(judge_pb2.DeleteContestInput(contest_id=contest_id))

    def delete_all_contests(self):
        for contest in self.get_contests():
            self.delete_contest(contest.id)

    def get_problems(self, contest_id):
        return self.client.ListProblems(judge_pb2.ListProblemsInput(contest_id=contest_id)).items

    def create_contest(self, name, domain, start, end, ioi=True):
        contest = contest_pb2.Contest(
            name=name,
            starts_at=timestamp_pb2.Timestamp(seconds=start),
            ends_at=timestamp_pb2.Timestamp(seconds=end),
            status=contest_pb2.Contest.Status.SCHEDULED,
            visibility=contest_pb2.Contest.Visibility.PRIVATE,
            participation_mode=contest_pb2.Contest.ParticipationMode.ONLINE,
            format=contest_pb2.Contest.Format.IOI if ioi else contest_pb2.Contest.Format.ICPC,
            domain=domain + '.eolymp.io'
        )
        res = self.client.CreateContest(judge_pb2.CreateContestInput(contest=contest))
        return res.contest_id

    def allow_upsolve(self, contest_id):
        return self.client.ConfigureScoring(judge_pb2.ConfigureScoringInput(contest_id=contest_id,
                                                                            scoring=contest_pb2.Contest.Scoring(
                                                                                allow_upsolving=True)))

    def get_participant(self, participant_id, contest_id):
        return self.client.DescribeParticipant(
            judge_pb2.DescribeParticipantInput(contest_id=contest_id, participant_id=participant_id)).participant

    def get_submissions(self, contest_id):
        def __get_submissions(offset, size):
            return self.client.ListSubmissions(
                request=judge_pb2.ListSubmissionsInput(offset=offset, size=size, contest_id=contest_id))

        return get_many(__get_submissions)

    def rejudge_problem(self, problem_id, contest_id):
        self.client.RetestProblem(judge_pb2.RetestProblemInput(contest_id=contest_id, problem_id=problem_id))

    def get_participants(self, contest_id):
        def __get_participants(offset, size):
            return self.client.ListParticipants(
                request=judge_pb2.ListParticipantsInput(contest_id=contest_id, offset=offset, size=size))

        return get_many(__get_participants)

    def download_sources(self, dest, contest_id):
        return download_sources_by_data(dest, self.get_contest(contest_id), self.get_problems(contest_id),
                                        self.get_participants(contest_id), self.get_submissions(contest_id))



from eolymp.atlas import statement_pb2, atlas_pb2
from eolymp.atlas.atlas_http import AtlasClient

from peolymp.abstract import AbstractAPI
from peolymp.utils import get_many


class AtlasAPI(AbstractAPI):

    def __init__(self, **kwargs):
        super(AtlasAPI, self).__init__(**kwargs)
        self.client = AtlasClient(self.http_client, url=self.get_url())

    def create_statement(self, prob_id, locale, title, link, source=""):
        s = statement_pb2.Statement(problem_id=prob_id, locale=locale, title=title, content="", download_link=link,
                                    format=statement_pb2.Statement.Format.TEX, source=source)
        return self.client.CreateStatement(atlas_pb2.CreateStatementInput(problem_id=prob_id, statement=s)).statement_id

    def update_statement(self, problem_id, statement):
        return self.client.UpdateStatement(
            atlas_pb2.UpdateStatementInput(problem_id=problem_id, statement_id=statement.id, statement=statement))

    def delete_statement(self, prob_id, statement_id):
        self.client.DeleteStatement(atlas_pb2.DeleteStatementInput(statement_id=statement_id, problem_id=prob_id))

    def get_statements(self, prob_id):
        return self.client.ListStatements(atlas_pb2.ListStatementsInput(problem_id=prob_id)).items

    def get_problems(self):
        def __get_problems(offset, size):
            return self.client.ListProblems(request=atlas_pb2.ListProblemsInput(offset=offset, size=size))

        return get_many(__get_problems)



import os

from setuptools import setup, find_packages

setup(
    name='peolymp',
    version=os.getenv("RELEASE_VERSION"),

    url='https://github.com/ukroi',
    author='Anton Tsypko',
    author_email='tsypko@oi.in.ua',

    packages=find_packages(),

    install_requires=[
        'eolymp',
        'requests',
        'protobuf'
    ],
)



from eolymp.community import community_pb2, member_pb2
from eolymp.community.community_http import CommunityClient
from eolymp.wellknown import expression_pb2

from peolymp.abstract import AbstractAPI
from peolymp.utils import get_many


class CommunityAPI(AbstractAPI):
    def __init__(self, **kwargs):
        super(CommunityAPI, self).__init__(**kwargs)
        self.client = CommunityClient(self.http_client, url=self.get_url())

    def get_member(self, member_id):
        return self.client.DescribeMember(community_pb2.DescribeMemberInput(member_id=member_id))

    def get_members(self):
        def get_members(offset, size):
            return self.client.ListMembers(request=community_pb2.ListMembersInput(offset=offset, size=size))

        return get_many(get_members)

    def describe_member(self, member_id):
        return self.client.DescribeMember(community_pb2.DescribeMemberInput(member_id=member_id)).member

    def delete_member(self, member_id):
        return self.client.RemoveMember(community_pb2.RemoveMemberInput(member_id=member_id))

    def delete_all_members(self):
        for member in self.get_members():
            self.delete_member(member.id)

    def make_members_incomplete(self):
        members = self.get_members()
        c = 0
        n = len(members)
        for member in members:
            c += 1
            m = member_pb2.Member(registered=False)
            member.registered = False
            if member.status == 2:
                continue
            print(member)
            print(self.client.UpdateMember(
                community_pb2.UpdateMemberInput(patch=[community_pb2.UpdateMemberInput.REGISTERED], member_id=member.id,
                                                member=m)))

            print((c * 100) // n)

    def find_member(self, user_id):
        member_exp = expression_pb2.ExpressionID(value=user_id)
        setattr(member_exp, 'is', 1)
        member_filter = community_pb2.ListMembersInput.Filter(user_id=[member_exp])
        m = self.client.ListMembers(community_pb2.ListMembersInput(filters=member_filter))
        if m.total == 1:
            return m.items[0]
        return None

    def create_manual_member(self, name, username, password, string_values=[], number_values=[], registered=False,
                             out_of_competition=False):
        identity = member_pb2.Member.Identity(email="fake+" + username + "@eolymp.com",
                                              issuer="https://api.eolymp.com/spaces/" + self.space_id,
                                              nickname=username, password=password)
        values = [member_pb2.Member.Value(attribute_key=pair[0], value_string=pair[1]) for pair in string_values] + \
                 [member_pb2.Member.Value(attribute_key=pair[0], value_number=pair[1]) for pair in number_values]
        member = member_pb2.Member(name=name, registered=registered, values=values, staffed=False,
                                   out_of_competition=out_of_competition, identities=[identity])
        result = self.client.AddMember(community_pb2.AddMemberInput(member=member))
        print(result)
        return result.member_id

    def create_manual_member_oi(self, name, username, password, grade, region, registered=False,
                                out_of_competition=False):
        return self.create_manual_member(name, username, password, string_values=[('region', region), ('name', name)],
                                         number_values=[('grade', int(grade))], registered=registered,
                                         out_of_competition=out_of_competition)

    def change_name(self, member_id, name):
        self.client.UpdateMember(community_pb2.UpdateMemberInput(patch=[community_pb2.UpdateMemberInput.NAME],
                                                                 member_id=member_id,
                                                                 member=member_pb2.Member(name=name)))

    def update_identity(self, member_id, identity_id, identity):
        self.client.UpdateMemberIdentity(
            community_pb2.UpdateMemberIdentityInput(member_id=member_id, identity_id=identity_id, identity=identity))



from eolymp.universe import universe_pb2, permission_pb2
from eolymp.universe.universe_http import UniverseClient

from peolymp.abstract import AbstractAPI


class UniverseAPI(AbstractAPI):
    def __init__(self, **kwargs):
        super(UniverseAPI, self).__init__(**kwargs)
        self.client = UniverseClient(self.http_client)

    def add_custom_admin_access(self, user_id):
        return self.client.GrantPermission(
            universe_pb2.GrantPermissionInput(space_id=self.space_id, role=permission_pb2.Permission.CUSTOM,
                                              user_id=user_id))



from eolymp.cognito import cognito_pb2
from eolymp.cognito.cognito_http import CognitoClient
from eolymp.wellknown import expression_pb2

from peolymp.abstract import AbstractAPI


class CognitoAPI(AbstractAPI):

    def __init__(self, **kwargs):
        super(CognitoAPI, self).__init__(**kwargs)
        self.client = CognitoClient(self.http_client)

    def find_user_by_nickname(self, nickname):
        s = expression_pb2.ExpressionString(value=nickname)
        setattr(s, 'is', 1)
        f = cognito_pb2.ListUsersInput.Filter(username=[s])
        users_input = cognito_pb2.ListUsersInput(filters=f)
        o = self.client.ListUsers(users_input)
        if len(o.items) == 0:
            return None
        return o.items[0]

    def find_user_by_id(self, user_id):
        return self.client.DescribeUser(cognito_pb2.DescribeUserInput(user_id=user_id)).user
